DECLARE SUB ANNOTATE (text$)
DECLARE SUB CDFIND (drive$, cdrom$, dir$)
DECLARE SUB CDPARAM (pixsiz, radius, projsamp, projlon, specline, scale)
DECLARE SUB EDGES (startrow, startcol, startline, startsamp)
DECLARE SUB LABEL (cdrom$, dir$, sline%, sample%, row%, column%, tile, pixelline, pixelsample)
DECLARE SUB LATLONG ()
DECLARE SUB LINESAMPLE ()
DECLARE SUB READWRITE (starttile, drive$, dir$, type$, startsamp, start, finish)
DECLARE SUB ROWCOL (sline%, sample%, row%, column%, tile, pixelline, pixelsample)
DECLARE SUB TILECREATE ()
DECLARE SUB TILEFIND ()
DECLARE FUNCTION VALUE! (text$, search$)

SCREEN 0: COLOR 15, 1  ' Bright white on blue

DO
 CLS
 PRINT
 PRINT "This program assists with using the Magellan CD-ROMs."
 PRINT
 PRINT "1. Find the latitude/longitude corresponding to a line/sample pair."
 PRINT "2. Find the line/sample corresponding to a latitude/longitude pair."
 PRINT "3. Find the image mosaic tile containing a line/sample pair."
 PRINT "4. Create a 1024x1024 full resolution image centered on a line/sample pair."
 PRINT "5. Quit"
 PRINT
 INPUT "Select a function (1,2,3,4 or 5): ", choice
 SELECT CASE choice
  CASE 1
   LATLONG
  CASE 2
   LINESAMPLE
  CASE 3
   TILEFIND
  CASE 4
   TILECREATE
  CASE 5
   COLOR 7, 0: CLS : END  ' Revert to white on black on exit
  CASE ELSE
 END SELECT
LOOP

' Insert an annotation string into a string of blanks,
' so as to maintain the character count unchanging.
'
SUB ANNOTATE (text$)
 junk$ = STRING$(60, " ")
 MID$(junk$, 1) = text$
 PRINT #3, junk$
END SUB

' Find the DOS drive letter, the CD name, and the directory name
' to use for the Magellan CD-ROM.
'
SUB CDFIND (drive$, cdrom$, dir$)
 PRINT
 INPUT "Your CD-ROM drive is drive (e,f,g,...): ", drive$
 PRINT
 drive$ = UCASE$(LEFT$(drive$, 1))
 dir$ = "dir " + drive$ + ":\*."
 SHELL dir$
 dir$ = dir$ + " > dir.txt"
 SHELL dir$
 OPEN "dir.txt" FOR INPUT AS #1
 LINE INPUT #1, dir$: LINE INPUT #1, dir$
 cdrom$ = RIGHT$(dir$, 7)
 CLOSE #1
 KILL "dir.txt"
 PRINT
 PRINT "Choose a directory (i.e. C160N347 - do **not** add a \)."
 PRINT "The program will use the file named BROWSE.IMG in that directory."
 PRINT
 PRINT "Use the image mosaic directory: " + drive$ + ":\";
 INPUT "", dir$
 dir$ = UCASE$(dir$)
END SUB

' Find the map projection parameters contained in the VICAR label
' on a Magellan BROWSE.IMG file.
'
SUB CDPARAM (pixsiz, radius, projsamp, projlon, specline, scale)
 PRINT
 CALL CDFIND(drive$, cdrom$, dir$)
 file$ = drive$ + ":\" + dir$ + "\BROWSE.IMG"
 PRINT file$
 PRINT
 OPEN file$ FOR BINARY AS #1
 data$ = STRING$(3072, " ")
 GET #1, 1, data$
 CLOSE #1
 pixsiz = VALUE(data$, "PIXSIZ")
 radius = VALUE(data$, "RADIUS")
 projsamp = VALUE(data$, "PROJSAMP")
 projlon = VALUE(data$, "PROJ_LON")
 radians = 3.14159# / 180!
 projlon = projlon * radians
 specline = VALUE(data$, "SPECLINE")
 IF radius = 0 THEN radius = 6051000#
 scale = radius / pixsiz
END SUB

' If the new tile would spill over the BROWSE.IMG edge, constrain
' it to start or end at the edge.
'
SUB EDGES (startrow, startcol, startline, startsamp)
 IF startrow < 1 THEN
  startrow = 1
  startline = 1
 END IF
 IF startrow = 7 THEN startline = 1
 IF startcol < 1 THEN
  startcol = 1
  startsamp = 1
 END IF
 IF startcol = 8 THEN startsamp = 1
END SUB

' Attach a SFDU label to the front of the new image tile, including
' a description of the source of the new tile.  The label size must be
' kept constant.
'
SUB LABEL (cdrom$, dir$, sline%, sample%, row%, column%, tile, pixelline, pixelsample)
 OPEN "newtile.img" FOR OUTPUT AS #3
 PRINT #3, "NJPL1I00PDS101050604            = SFDU_LABEL"
 PRINT #3, "RECORD_TYPE                     = FIXED_LENGTH"
 PRINT #3, "RECORD_BYTES                    = 1024"
 PRINT #3, "FILE_RECORDS                    = 1026"
 PRINT #3, "LABEL_RECORDS                   = 2"
 PRINT #3, "^IMAGE                          = 3"
 PRINT #3, "OBJECT                          = IMAGE"
 PRINT #3, "  LINES                         = 1024"
 PRINT #3, "  LINE_SAMPLES                  = 1024"
 PRINT #3, "  SAMPLE_BITS                   = 8"
 PRINT #3, "END_OBJECT"
 PRINT #3,
 text$ = "Magellan CD-ROM disk = " + cdrom$
 CALL ANNOTATE(text$)
 text$ = "Tile created from mosaic " + dir$ + "."
 CALL ANNOTATE(text$)
 text$ = "Centered on BROWSE.IMG line/sample =" + STR$(sline%) + " /" + STR$(sample%)
 CALL ANNOTATE(text$)
 text$ = "Row =" + STR$(row%) + "   Column =" + STR$(column%) + "   Tile =" + STR$(tile)
 CALL ANNOTATE(text$)
 text$ = "Line/sample in tile =" + STR$(pixelline) + " /" + STR$(pixelsample)
 CALL ANNOTATE(text$)
 PRINT #3,
 PRINT #3, "END"
 PRINT #3, STRING$(1311, " ");
END SUB

' Find lat/long for a line/sample pair in BROWSE.IMG.
'
SUB LATLONG
 CLS
 PRINT "This program finds the latitude/longitude corresponding to"
 PRINT "pixel line/sample pairs from Magellan browse image mosaics."
 CALL CDPARAM(pixsiz, radius, projsamp, projlon, specline, scale)
 PRINT "Value of line between 1 and 896, sample between 1 and 1024."
 PRINT
 PRINT "Enter line = -1 to stop."
 PRINT
 DO
  INPUT "Enter line: ", sline
  IF sline = -1 THEN EXIT SUB
  INPUT "Enter sample: ", sample
  sline = sline * 8
  sample = sample * 8
  lat = (specline - sline + .5) / scale
  lon = (sample - projsamp - .5) / (COS(lat) * scale) + projlon
  radians = 3.14159# / 180!
  lat = lat / radians
  lon = lon / radians
  IF lon >= 360 THEN lon = lon - 360
  IF lon < 0 THEN lon = lon + 360
  PRINT "latitude: "; lat, "longitude: "; lon
 LOOP
END SUB

' Find line/sample in BROWSE.IMG for a lat/long pair.
'
SUB LINESAMPLE
 CLS
 PRINT "This program finds the pixel line/sample pair corresponding to"
 PRINT "a chosen latitude/longitude for Magellan browse image mosaics."
 CALL CDPARAM(pixsiz, radius, projsamp, projlon, specline, scale)
 PRINT "Value of latitude between 90 and -90, longitude between 0 and 360."
 PRINT
 PRINT "Enter latitude = 100 to stop."
 PRINT
 DO
  INPUT "Enter latitude: ", lat
  IF lat = 100 THEN EXIT SUB
  INPUT "Enter longitude: ", lon
  radians = 3.14159# / 180!
  lat = lat * radians
  lon = lon * radians
  sline = specline - lat * scale + .5
  sample = projsamp + ((lon - projlon) * scale * COS(lat)) + .5
  aa% = sline / 8
  bb% = sample / 8
  PRINT "line: "; aa%, "sample: "; bb%
 LOOP
END SUB

' Reads the original tiles from the CD-ROM, assembles new tile lines,
' and writes the new tile onto the hard disk.  Does not read
' unnecessary tiles.
'
SUB READWRITE (starttile, drive$, dir$, type$, startsamp, start, finish)
 starttile$ = MID$(STR$(starttile), 2)
 nexttile$ = MID$(STR$(starttile + 1), 2)
 IF LEN(starttile$) = 1 THEN starttile$ = "0" + starttile$
 IF LEN(nexttile$) = 1 THEN nexttile$ = "0" + nexttile$
 starttile$ = drive$ + ":\" + dir$ + "\" + type$ + "F" + starttile$ + ".img"
 nexttile$ = drive$ + ":\" + dir$ + "\" + type$ + "F" + nexttile$ + ".img"
 SHELL "copy " + starttile$ + " start.img"
 IF startsamp > 1 THEN SHELL "copy " + nexttile$ + " next.img"
 OPEN "start.img" FOR BINARY AS #1
 OPEN "next.img" FOR BINARY AS #2
 startdata$ = STRING$(1024, " ")
 nextdata$ = startdata$
 out$ = startdata$
 sline = start
 DO WHILE sline < finish
  position = 2048 + ((sline - 1) * 1024) + 1
  GET #1, position, startdata$
  GET #2, position, nextdata$
  out$ = RIGHT$(startdata$, 1025 - startsamp) + LEFT$(nextdata$, startsamp - 1)
  PRINT #3, out$;
  sline = sline + 1
 LOOP
 CLOSE #1: CLOSE #2
 KILL "start.img": KILL "next.img"
END SUB

' Finds which tile row/column and which line/sample in that tile
' corresponds to a line/sample pair in BROWSE.IMG.
'
SUB ROWCOL (sline%, sample%, row%, column%, tile, pixelline, pixelsample)
 row% = (sline% / 128) - .50001 + 1
 column% = (sample% / 128) - .50001 + 1
 tile = column% + (row% - 1) * 8
 pixelline = (sline% * 8) - (row% - 1) * 1024
 pixelsample = (sample% * 8) - (column% - 1) * 1024
END SUB

' Creates a new mosaic tile centered on a BROWSE.IMG line/sample pair.
'
SUB TILECREATE
 CLS
 PRINT "This program creates a new 1024x1024 mosaic tile centered on a line/sample"
 PRINT "pair in the browse image."
 PRINT
 PRINT "The new tile will be named NEWTILE.IMG and be placed in the *same*"
 PRINT "directory as this program.  It will *overwrite* any other file with the"
 PRINT "same name.  Two temporary files (start.img and next.img) will also"
 PRINT "erase any other files with their names."
 CALL CDFIND(drive$, cdrom$, dir$)
 type$ = LEFT$(dir$, 2)
 IF LEFT$(type$, 1) = "F" THEN type$ = LEFT$(type$, 1)
 PRINT
 PRINT "Enter the line/sample pair on which you want the new tile centered."
 PRINT "Line between 1 and 896, sample between 1 and 1024."
 PRINT
 PRINT "(Enter line = -1 to quit.)"
 PRINT
 INPUT "Enter line: ", sline%
   IF sline% < 1 OR sline% > 896 THEN EXIT SUB
 INPUT "Enter sample: ", sample%
   IF sample% < 1 OR sample% > 1024 THEN EXIT SUB
 CALL ROWCOL(sline%, sample%, row%, column%, tile, pixelline, pixelsample)
 PRINT "Row: "; row%; "  Column: "; column%; "  Tile: "; tile
 PRINT "Line in tile: "; pixelline; "  Sample in tile: "; pixelsample
 PRINT
 startrow = row%
 startcol = column%
 IF pixelline < 512 THEN startrow = row% - 1
 IF pixelsample < 512 THEN startcol = column% - 1
 startline = pixelline + 512
 startsamp = pixelsample + 512
 IF startline > 1024 THEN startline = startline - 1024
 IF startsamp > 1024 THEN startsamp = startsamp - 1024
 CALL EDGES(startrow, startcol, startline, startsamp)
 starttile = startcol + (startrow - 1) * 8
 start = startline
 finish = 1025
 CALL LABEL(cdrom$, dir$, sline%, sample%, row%, column%, tile, pixelline, pixelsample)
 CALL READWRITE(starttile, drive$, dir$, type$, startsamp, start, finish)
 IF startline > 1 THEN
  starttile = starttile + 8
  start = 1
  finish = startline
  CALL READWRITE(starttile, drive$, dir$, type$, startsamp, start, finish)
 END IF
 CLOSE
END SUB

' Find the tile row/column and line/sample for a BROWSE.IMG line/sample.
'
SUB TILEFIND
 CLS
 PRINT "This program finds the tile which contains a browse image line/sample pair,"
 PRINT "the tile's row and column in the mosaic, and the line/sample within the tile."
 PRINT
 PRINT "(Enter line = -1 to quit.)"
 PRINT
 DO
  INPUT "Enter line: ", sline%
  IF sline% < 1 THEN EXIT SUB
  INPUT "Enter sample: ", sample%
  CALL ROWCOL(sline%, sample%, row%, column%, tile, pixelline, pixelsample)
  PRINT "Row: "; row%; "  Column: "; column%; "  Tile: "; tile
  PRINT "Line in tile: "; pixelline; "  Sample in tile: "; pixelsample
  PRINT
 LOOP
END SUB

' Find the numeric value of a parameter in the VICAR label.
'
FUNCTION VALUE (text$, search$)
  position = INSTR(text$, search$)
  IF position THEN
    junk$ = MID$(text$, position + LEN(search$) + 1)
    VALUE = VAL(junk$)
  ELSE
    VALUE = 0
  END IF
END FUNCTION

